/*       Description: ODE Solver Model                 *
*                                                      *
*        Author: Shaylesh Mehta                        *
*                                                      *
*        Email: skmehta@connectmail.carleton.ca        *
*                                                      *
*        Date: February 7th 2002                       *
*******************************************************/

/* include files */

#include "message.h"
#include "mainsimu.h"
#include "ODEModel.h"
#include "ODESolver.h"
#include "ODEs.h"

/* public functions */
// need a seperate one of these for each ODE system in the vhdl design


ODEModel :: ODEModel( const string &name) 
: Atomic (name)
, exo_1(addInputPort ("exo_1"))   //constructors for input and output ports, exogenous to ODE model
, y(addInputPort("y"))            // here nVar = 1  
, dy(addInputPort("dy"))

{
}


/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: 
********************************************************************/
Model &ODEModel::initFunction()
{
    
    yscal=(Value *)calloc(nVar,sizeof(Value)); //scaler for accuracy control
    _y=(Value *)calloc(nVar,sizeof(Value));
    _dy=(Value *)calloc(nVar,sizeof(Value));
    coupled_inputs=(Value *)calloc(nExog,sizeof(Value));
    
    for(i=0;i<nExog;i++)// initial value for the buffers of inputs to the model
    {
        coupled_inputs[i] = 0;
    }
    
    //set initial values for _y and _dy here

    _y=0;
    _dy=0;


    holdIn(passive,0);
    return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &ODEModel::externalFunction( const ExternalMessage &msg )
{
    if (msg.port()==exo_1) { //if inputs to DE system change update buffered value    
    
        coupled_inputs[0]=msg.value; // assign coupled inputs to vector here
    }
    // no state transition should take place here, should remain in active wait state
    return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &ODEModel::internalFunction( const InternalMessage & )
{
    //call derivative function here
    
    //calculate inital derrivatives
    ODE_2(x,_y,_dy,coupled_inputs);  // ***** change this for each ODE system in design

    for(i=0;i<nVar;i++)//calculate scaler for error
        yscal[i]=fabs(_y[i])+fabs(_dy[i]*h)+TINY;
    
    //take step, ***** change ODE_2 for each ODE system in design 
    if(rkstep(_y,_dy,nVar,&x,h,epsilon,yscal,&hdid,&hnext,ODE_2,coupled_inputs)!=OK)
    {
        printf("rkstep failed to take a step!\n");
        fprintf(stderr,"rkstep failed to take a step!\n");
        return *this;
    }

    if (hnext<hmin) {
        printf("step size below hmin in ODEModel\n");
        fprintf(stderr,"step size below hmin in ODEModel\n");
        return *this;
    }
    
    h=hnext;                      // 1 ms = 1us in simulator
    idlePeriod=Time((int)(hdid*Timebase_Scaler)); //time base, have to cast to int because of integer time
    holdIn(active,idlePeriod);   //wait and output results at appropriate time 
    
    return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &ODEModel::outputFunction( const InternalMessage &msg )
{
        if(state()==active) //change outputs accoring to number of variables in ODE system
        {
            this->sendOutput(msg.time(),y,*(_y[0]));   // here nVar=1
            this->sendOutput(msg.time(),dy,*(_dy[0]));
            holdIn(passive,0);
        }

        return *this ;
}

