
/*******************************************************************
*
*  DESCRIPTION: VHDL ODE Solver Model 
*
*  AUTHOR: Shaylesh Mehta
*
*  EMAIL: skmehta@connectmail.carleton.ca
*         
*
*  DATE: February 7th 2002
*
*******************************************************************/

// prevent multiple inclusion with preprocessor directive
#ifndef __ODE_SOLVER_H 
#define __ODE_SOLVER_H

#define Timebase_Scaler 1000

#include "atomic.h"

//need to have a seperate ODEModel class for each ODE system in the 
//vhdl design

class ODEModel : public Atomic
{
public:
    ODEModel( const string &name = "ODEModel" );//Default constructor

    virtual string className() const ;

protected:
    Model &internalFunction(const InternalMessage &) ;
    Model &initFunction() ;
    Model &externalFunction(const ExternalMessage &) ;
    Model &outputFunction(const InternalMessage &) ;
private:

    //port declarations, one input, one output

    const Port &exo_1;
    Port &y,&dy;
    
    int nVar = 1;
    int nExog = 1;
    int i;
    Value epsilon = 1E-3;

    Value *_y
    Value *_dy;
    Value *yscal;
    Value *coupled_inputs;
    
    double hnext, hdid, h, x;

    Time idlePeriod;
    

};//class Signal

// ** inline ** // 
inline
string ODEModel::className() const
{
	return "ODEModel" ;
}


#endif //__ODE_SOLVER_H

